﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MontoyaTech.Parallel.Net
{
    public static class ParallelTaskExtensions
    {
        /// <summary>
        /// Process a list of items in parallel and returns the results. The results will be out of order.
        /// </summary>
        /// <typeparam name="InputT">Input type</typeparam>
        /// <typeparam name="ResultT">Result type</typeparam>
        /// <param name="list">The list of items to process in parallel</param>
        /// <param name="func">The function to take an input and return a result</param>
        /// <param name="maxTasks">The max tasks to run in parallel if specified. Default is null. If null the number of items will be the number of tasks</param>
        /// <param name="timeout">If set a timeout will be thrown if this doesnt complete in time. Default is null</param>
        /// <param name="throwOnTimeout">Whether or not to throw an exception on timeout. Default is true</param>
        /// <param name="bubbleExceptions">If true, rethrows any exceptions from the running tasks. Default is true.</param>
        /// <returns>A list of results from the items processed.</returns>
        public static List<ResultT> ForEachParallel<InputT, ResultT>(this IList<InputT> list, Func<InputT, ResultT> func, int? maxTasks = null, TimeSpan? timeout = null, bool throwOnTimeout = true, bool bubbleExceptions = true)
        {
            return ParallelTask.ForEach<InputT, ResultT>(list, func, maxTasks, timeout, throwOnTimeout, bubbleExceptions);
        }

        /// <summary>
        /// Processes a list of items in parallel and runs an action against them.
        /// </summary>
        /// <typeparam name="InputT">Input type</typeparam>
        /// <param name="list">The list of items to process in parallel</param>
        /// <param name="action">The action to take an input and perform an operation on it</param>
        /// <param name="maxTasks">The max tasks to run in parallel if specified. Default is null. If null the number of items will be the number of tasks</param>
        /// <param name="timeout">If set a timeout will be thrown if this doesnt complete in time. Default is null</param>
        /// <param name="throwOnTimeout">Whether or not to throw an exception on timeout. Default is true</param>
        /// <param name="bubbleExceptions">If true, rethrows any exceptions from the running tasks. Default is true.</param>
        public static void ForEachParallel<InputT>(this IList<InputT> list, Action<InputT> action, int? maxTasks = null, TimeSpan? timeout = null, bool throwOnTimeout = true, bool bubbleExceptions = true)
        {
            ParallelTask.ForEach<InputT>(list, action, maxTasks, timeout, throwOnTimeout, bubbleExceptions);
        }
    }
}
